/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URL;

import org.apache.commons.io.FileUtils;
import org.ow2.petals.jbi.management.Context;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.util.LoggingUtil;

/**
 * This task copy entity (component, shared lib, service assembly) package into
 * the work directory<br/>
 * Inputs : archive {@link URI} <br/>
 * Outputs : {@link File} pointing on unzipped archive <br/>
 * The task have got a non-robust approach : it stops at the first subtask
 * failed<br />
 * and have the undo capability.<br />
 * 
 * @author ofabre - EBM Websourcing
 * @author Frederic Gardes - eBM Websourcing
 */
public class CopyArchiveToWorkDirTask extends AbstractFileManipulationTask {

    public CopyArchiveToWorkDirTask(LoggingUtil log, String petalsInstallDirectory) {
        super(log, petalsInstallDirectory);
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.jbi.management.Task#execute(org.ow2.petals.jbi.management.Context)
     */
    public void execute(Context context) throws Exception {
        URL archiveURL = context.getArchiveUrl();
        File archive = this.copyEntityPackage(archiveURL);
        context.setArchiveUrl(archive.toURI().toURL());
    }

    private File copyEntityPackage(URL archiveURL) throws PetalsException {
        File archiveFile = new File(archiveURL.getPath());
        File destFile = new File(this.getWorkDirectory(), archiveFile.getName());
        try {
            FileUtils.copyFile(archiveFile, destFile);
        } catch (IOException e) {
            throw new PetalsException("Can't copy entity package to work directory", e);
        }
        
        return destFile;
    }

    /*
     * (non-Javadoc)
     * @see org.ow2.petals.jbi.management.Task#undo(org.ow2.petals.jbi.management.Context)
     */
    public void undo(Context context) throws Exception {
        URL archiveURL = context.getArchiveUrl();
        if (archiveURL != null) {
            try {
                FileUtils.forceDelete(new File(archiveURL.getPath()));
            } catch (IOException e) {
                this.log.warning("Failed to clean the copied archive '" + archiveURL.getPath()
                        + "'");
            }
        }
    }
}
