/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.jbi.management.task.installation.install.component;

import org.ow2.petals.container.lifecycle.SharedLibraryLifeCycle;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.descriptor.original.generated.Component.SharedLibrary;
import org.ow2.petals.jbi.management.Task;
import org.ow2.petals.jbi.management.Context;
import org.ow2.petals.jbi.management.admin.AdminService;
import org.ow2.petals.kernel.api.server.PetalsException;

/**
 * This task checks installed shared library for component and register this
 * latter. A shared librairy must be installed before a component can be
 * installed <br/>
 * Inputs : {@link Jbi} <br/>
 * 
 * @author ofabre
 * 
 */
public class RegisterComponentInSLTask implements Task {

    private final AdminService adminService;

    public RegisterComponentInSLTask(AdminService adminService) {
        super();
        this.adminService = adminService;
    }

    public void execute(Context context) throws Exception {
        Jbi descriptor = context.getDescriptor();

        if (descriptor.getComponent().getSharedLibraryList() != null) {
            for (SharedLibrary sl : descriptor.getComponent().getSharedLibraryList()) {
                SharedLibraryLifeCycle saLC = this.adminService.getSharedLibraryByName(sl
                        .getContent() + "-" + sl.getVersion());
                if (saLC == null) {
                    throw new PetalsException("The component depends on the Shared Library '"
                            + sl.getContent() + "' with version '" + sl.getVersion() + "' which is not installed");
                }
                saLC.registerComponent(descriptor.getComponent().getIdentification().getName());
            }
        }
    }

    public void undo(Context context) throws Exception {
        Jbi descriptor = context.getDescriptor();

        if (descriptor.getComponent().getSharedLibraryList() != null) {
            for (SharedLibrary sl : descriptor.getComponent().getSharedLibraryList()) {
                SharedLibraryLifeCycle saLC = this.adminService.getSharedLibraryByName(sl
                        .getContent() + "-" + sl.getVersion());
                if (saLC != null) {
                    saLC.unregisterComponent(descriptor.getComponent().getIdentification()
                            .getName());
                }
            }
        }

    }

}
