/**
 * PETALS - PETALS Services Platform. Copyright (c) 2009 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 */
package org.ow2.petals.jbi.messaging.routing.module;

import java.util.Map;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.jbi.component.context.ComponentContext;
import org.ow2.petals.jbi.messaging.endpoint.ServiceEndpoint;
import org.ow2.petals.jbi.messaging.exchange.MessageExchange;
import org.ow2.petals.jbi.messaging.routing.RouterService;
import org.ow2.petals.jbi.messaging.routing.RoutingException;
import org.ow2.petals.kernel.configuration.ConfigurationService;
import org.ow2.petals.kernel.configuration.DomainConfiguration.DomainMode;
import org.ow2.petals.transport.Transporter;
import org.ow2.petals.transport.util.TransportSendContext;
import org.ow2.petals.util.LoggingUtil;

/**
 * @author rnaudin
 * 
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = SenderModule.class))
public class TransportResolverModule implements SenderModule {

    /**
     * Logger wrapper.
     */
    protected LoggingUtil log;

    /**
     * The logger.
     */
    @Monolog(name = "logger")
    protected Logger logger;

    /**
     * The Topology service fractal component
     */
    @Requires(name = "configuration", signature = org.ow2.petals.kernel.configuration.ConfigurationService.class)
    protected ConfigurationService configurationService;

    private String localContainerName;

    private DomainMode domainMode;

    private short attempt;

    private int delay;

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ow2.petals.jbi.messaging.routing.module.SenderModule#electDestinations
     * (java.util.Map,
     * org.ow2.petals.jbi.component.context.ComponentContextImpl,
     * org.ow2.petals.jbi.messaging.exchange.MessageExchangeImpl)
     */
    public void electEndpoints(Map<ServiceEndpoint, TransportSendContext> electedDestinations,
            ComponentContext sourceComponentContext, MessageExchange exchange)
            throws RoutingException {
        this.log.call();

        // TODO handle transport selection by the message Exchange properties or
        // by the topology...

        for (TransportSendContext transportSendContext : electedDestinations.values()) {
            if (this.localContainerName.equals(transportSendContext.destination.getContainerName())) {
                transportSendContext.transport = Transporter.LOCAL_FRACTAL_TRANSPORTER;
            } else {
                transportSendContext.transport = Transporter.TCP_FRACTAL_TRANSPORTER;
            }

            if (DomainMode.STANDALONE.equals(this.domainMode)) {
                transportSendContext.attempt = 1;
            } else {
                if (exchange.getProperty(RouterService.PROPERTY_ROUTER_SEND_ATTEMPT) != null) {
                    transportSendContext.attempt = Short.valueOf((String) exchange
                            .getProperty(RouterService.PROPERTY_ROUTER_SEND_ATTEMPT));
                } else {
                    transportSendContext.attempt = this.attempt;
                }

                if (exchange.getProperty(RouterService.PROPERTY_ROUTER_SEND_DELAY) != null) {
                    transportSendContext.delay = Integer.valueOf((String) exchange
                            .getProperty(RouterService.PROPERTY_ROUTER_SEND_DELAY));
                } else {
                    transportSendContext.delay = this.delay;
                }
            }
        }
    }

    /**
     * Start the Fractal component
     */
    @LifeCycle(on = LifeCycleType.START)
    protected void start() {
        this.log = new LoggingUtil(this.logger);
        this.log.call();

        this.localContainerName = this.configurationService.getContainerConfiguration().getName();
        this.domainMode = this.configurationService.getDomainConfiguration().getMode();
        this.attempt = this.configurationService.getContainerConfiguration().getRouterSendAttempt();
        this.delay = this.configurationService.getContainerConfiguration().getRouterSendDelay();
    }

    /**
     * Stop the Fractal component
     */
    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() throws RoutingException {
        this.log.call();
    }

}
