/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id: RouterImpl.java,v 1.2 2005/07/22 10:24:27 alouis Exp $
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.jbi.messaging.routing.module.endpoint;

import org.ow2.petals.jbi.messaging.endpoint.ServiceEndpoint;


/**
 * A handler of a weighted endpoint.
 * 
 * @author rnaudin
 * 
 */
public class WeightedEndpoint implements Comparable<WeightedEndpoint> {

    /**
     * Localization of the endpoint
     * 
     * @author rnaudin
     * 
     */
    public static enum Localization {
        LOCAL, REMOTE_ACTIVE, REMOTE_INACTIVE
    }

    /**
     * The endpoint instance
     */
    public ServiceEndpoint endpoint;

    /**
     * The weight of the endpoint
     */
    public Float weight;

    /**
     * The localization of the endpoint
     */
    public Localization localization;

    public WeightedEndpoint(final ServiceEndpoint endpoint, final float weighting,
            Localization localization) {
        super();
        this.endpoint = endpoint;
        this.weight = new Float(weighting);
        this.localization = localization;
    }

    public WeightedEndpoint(final ServiceEndpoint endpoint, Localization localization) {
        this(endpoint, 0.0f, localization);
    }

    /**
     * The comparison is reversed
     */
    public int compareTo(final WeightedEndpoint comparatedObject) {
        return -this.weight.compareTo(comparatedObject.weight);
    }
}
