/**
 * PETALS: PETALS Services Platform Copyright (C) 2009 EBM WebSourcing
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 * 
 * Initial developer(s): EBM WebSourcing
 */
package org.ow2.petals.ws;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.util.monolog.api.Logger;
import org.ow2.petals.kernel.api.server.PetalsException;
import org.ow2.petals.kernel.configuration.ContainerConfiguration;
import org.ow2.petals.kernel.configuration.DomainConfiguration;
import org.ow2.petals.kernel.configuration.JndiConfiguration;
import org.ow2.petals.kernel.configuration.SubDomainConfiguration;
import org.ow2.petals.kernel.configuration.ContainerConfiguration.RegistryMode;
import org.ow2.petals.kernel.ws.api.PEtALSWebServiceException;
import org.ow2.petals.kernel.ws.api.TopologyService;
import org.ow2.petals.kernel.ws.api.to.ContainerInformation;
import org.ow2.petals.kernel.ws.api.to.DomainInformation;
import org.ow2.petals.kernel.ws.api.to.JmxServiceInformation;
import org.ow2.petals.kernel.ws.api.to.JndiInformation;
import org.ow2.petals.kernel.ws.api.to.RegistryServiceInformation;
import org.ow2.petals.kernel.ws.api.to.SubdomainInformation;
import org.ow2.petals.kernel.ws.api.to.TransportServiceInformation;
import org.ow2.petals.kernel.ws.api.to.WebServiceServiceInformation;
import org.ow2.petals.kernel.ws.api.to.SubdomainInformation.SubdomainMode;
import org.ow2.petals.tools.ws.KernelWebService;
import org.ow2.petals.util.LoggingUtil;
import org.ow2.petals.util.StringHelper;
import org.ow2.petals.ws.adapters.TopologyConfigurationToTopologyInformation;

/**
 * @author chamerling - eBM WebSourcing
 * 
 */
@FractalComponent
@Provides(interfaces = { @Interface(name = "webservice", signature = TopologyService.class),
        @Interface(name = "service", signature = KernelWebService.class) })
public class TopologyServiceImpl implements TopologyService, KernelWebService {

    @Monolog(name = "logger")
    private Logger logger;

    private LoggingUtil log;

    @org.objectweb.fractal.fraclet.annotation.annotations.Service(name = "component")
    private Component component;

    @Requires(name = "topology", signature = org.ow2.petals.communication.topology.TopologyService.class)
    private org.ow2.petals.communication.topology.TopologyService topologyService;

    @LifeCycle(on = LifeCycleType.START)
    protected void start() {
        this.log = new LoggingUtil(this.logger);
        this.log.debug("Starting...");
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() {
        this.log.debug("Stopping...");
    }

    /**
     * {@inheritDoc}
     */
    public Component getComponent() {
        return this.component;
    }

    /**
     * {@inheritDoc}
     */
    public JndiInformation getJndiInformation() throws PEtALSWebServiceException {
        JndiInformation result = null;
        DomainConfiguration domainConfiguration = this.topologyService.getDomainConfiguration();

        if (domainConfiguration == null) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't retrieve the domain configuration.");
            throw new PEtALSWebServiceException(errorSB.toString());
        }
        JndiConfiguration jndiConfiguration = domainConfiguration.getJndiConfiguration();

        if (jndiConfiguration == null) {
            return result;
        }

        if (StringHelper.isNullOrEmpty(jndiConfiguration.getJndiFactory())) {
            return result;
        }

        if ((jndiConfiguration.getJndiProviderUrl() == null)
                || (StringHelper.isNullOrEmpty(jndiConfiguration.getJndiProviderUrl()
                        .toASCIIString()))) {
            return result;
        }

        result = new JndiInformation();

        result.setProviderUrl(jndiConfiguration.getJndiProviderUrl().toString());
        result.setFactory(jndiConfiguration.getJndiFactory());

        if (!StringHelper.isNullOrEmpty(jndiConfiguration.getJndiSecurityPrincipal())) {
            result.setSecurityPrincipal(jndiConfiguration.getJndiSecurityPrincipal());
        }

        if (!StringHelper.isNullOrEmpty(jndiConfiguration.getJndiSecurityCredentials())) {
            result.setSecurityPrincipal(jndiConfiguration.getJndiSecurityCredentials());
        }

        result.setBatchSize(domainConfiguration.getJndiConfiguration().getJndiBatchSize());
        result.setPoolSize(domainConfiguration.getJndiConfiguration().getJndiPoolSize());

        return result;
    }

    /**
     * {@inheritDoc}
     */
    public void addContainerInformation(ContainerInformation addedContainer, String subdomainName)
            throws PEtALSWebServiceException {
        this.addContainerInformation(addedContainer, subdomainName, true);
    }

    private void addContainerInformation(ContainerInformation addedContainer, String subdomainName,
            boolean validateTopology) throws PEtALSWebServiceException {
        ContainerConfiguration addedContainerConfiguration = null;

        if (!this.topologyService.hasValidLocalContainerDynamicTopologyConfiguration()) {
            throw new PEtALSWebServiceException(
                    "No dynamic configuration found on the target container");
        }

        if (addedContainer == null) {
            StringBuilder errorSB = new StringBuilder();
            errorSB.append("Can't add a null container to the topology");
            throw new PEtALSWebServiceException(errorSB.toString());
        }

        try {
            // Check if the subdomain has been already registered...
            boolean existingSubdomain = false;
            Set<SubDomainConfiguration> subdomainsConfigurations = this.topologyService
                    .getSubDomainsConfiguration();
            for (SubDomainConfiguration subdomainConfiguration : subdomainsConfigurations) {
                if (subdomainConfiguration.getName().equals(subdomainName)) {
                    existingSubdomain = true;
                    break;
                }
            }

            if (!existingSubdomain) {
                StringBuilder errorSB = new StringBuilder();
                errorSB.append("You must register a new subdomain with name : " + subdomainName);
                throw new PEtALSWebServiceException(errorSB.toString());
            }

            addedContainerConfiguration = new ContainerConfiguration();
            addedContainerConfiguration.setHost(addedContainer.getHost());
            addedContainerConfiguration.setName(addedContainer.getName());
            addedContainerConfiguration.setDescription(addedContainer.getDescription());
            addedContainerConfiguration.setUser(addedContainer.getUser());
            addedContainerConfiguration.setPassword(addedContainer.getPassword());
            addedContainerConfiguration.setSubdomainName(subdomainName);

            RegistryServiceInformation registryServiceInformation = addedContainer
                    .getRegistryService();
            if (registryServiceInformation != null) {
                addedContainerConfiguration.setRegistryPort(registryServiceInformation.getPort());
            } else {
                throw new PEtALSWebServiceException(
                        "Can't find remote container registry configuration");
            }

            JmxServiceInformation jmxServiceInformation = addedContainer.getJmxService();
            if (jmxServiceInformation != null) {
                addedContainerConfiguration.setJmxRMIConnectorPort(jmxServiceInformation
                        .getRmiPort());
            } else {
                throw new PEtALSWebServiceException("Can't find remote container JMX configuration");
            }

            TransportServiceInformation transportServiceInformation = addedContainer
                    .getTransportService();
            if (transportServiceInformation != null) {
                addedContainerConfiguration.setTCPPort(transportServiceInformation.getTcpPort());
            } else {
                throw new PEtALSWebServiceException(
                        "Can't find remote container transporter configuration");
            }

            WebServiceServiceInformation webServiceServiceInformation = addedContainer
                    .getWebserviceService();
            if (webServiceServiceInformation != null) {
                addedContainerConfiguration.setWebservicePort(webServiceServiceInformation
                        .getPort());
                addedContainerConfiguration.setWebservicePrefix(webServiceServiceInformation
                        .getPrefix());
            } else {
                throw new PEtALSWebServiceException(
                        "Can't find remote container webservice service configuration");
            }

            switch (addedContainer.getNodeType()) {
                case STANDALONE:
                    addedContainerConfiguration.setRegistryMode(RegistryMode.STANDALONE);
                    break;
                case MASTER:
                    addedContainerConfiguration.setRegistryMode(RegistryMode.MASTER);
                    break;
                case SLAVE:
                    addedContainerConfiguration.setRegistryMode(RegistryMode.SLAVE);
                    break;
                case PEER:
                    addedContainerConfiguration.setRegistryMode(RegistryMode.PEER);
                    break;
                default:
                    StringBuilder errorSB = new StringBuilder();
                    errorSB.append("Try to add a node with an invalid node type");
                    throw new PEtALSWebServiceException(errorSB.toString());
            }

            this.topologyService.addContainerConfiguration(addedContainerConfiguration);
        } catch (PetalsException ex) {
            throw new PEtALSWebServiceException(ex);
        }
    }

    public void removeContainerInformation(String containerName) throws PEtALSWebServiceException {
        if (!this.topologyService.hasValidLocalContainerDynamicTopologyConfiguration()) {
            throw new PEtALSWebServiceException(
                    "No dynamic configuration found on the target container");
        }

        try {
            ContainerConfiguration removedContainerConfiguration = new ContainerConfiguration();
            removedContainerConfiguration.setName(containerName);
            this.topologyService.removeContainerConfiguration(removedContainerConfiguration);
        } catch (PetalsException ex) {
            throw new PEtALSWebServiceException(ex);
        }
    }

    /**
     * {@inheritDoc}
     */
    public List<ContainerInformation> getAllContainerInformation() throws PEtALSWebServiceException {
        Set<ContainerConfiguration> configs = this.topologyService.getContainersConfiguration(null);

        List<ContainerInformation> result = new ArrayList<ContainerInformation>();
        for (ContainerConfiguration cc : configs) {
            try {
                result.add(TopologyConfigurationToTopologyInformation
                        .containerConfigurationToContainerInformation(cc));
            } catch (PetalsException ex) {
                throw new PEtALSWebServiceException(
                        "Can't transform ContainerConfiguration to ContainerInformation");
            }
        }

        return result;
    }

    /**
     * {@inheritDoc}
     */
    public List<ContainerInformation> getMasterContainerInformation()
            throws PEtALSWebServiceException {
        Set<ContainerConfiguration> configs = this.topologyService.getContainersConfiguration(null);

        List<ContainerInformation> result = new ArrayList<ContainerInformation>();
        for (ContainerConfiguration cc : configs) {
            try {
                if (cc.isMaster()) {
                    result.add(TopologyConfigurationToTopologyInformation
                            .containerConfigurationToContainerInformation(cc));
                }
            } catch (PetalsException ex) {
                throw new PEtALSWebServiceException(
                        "Can't transform ContainerConfiguration to ContainerInformation");
            }
        }

        return result;
    }

    /**
     * {@inheritDoc}
     */
    public List<ContainerInformation> getAllSlaveContainerInformationForLocalSubdomain()
            throws PEtALSWebServiceException {
        Set<ContainerConfiguration> configs = this.topologyService.getContainersConfiguration(null);

        List<ContainerInformation> result = new ArrayList<ContainerInformation>();
        for (ContainerConfiguration cc : configs) {
            try {
                if (cc.isSlave()) {
                    result.add(TopologyConfigurationToTopologyInformation
                            .containerConfigurationToContainerInformation(cc));
                }
            } catch (PetalsException ex) {
                throw new PEtALSWebServiceException(
                        "Can't transform ContainerConfiguration to ContainerInformation");
            }
        }

        return result;
    }

    /**
     * {@inheritDoc}
     */
    public List<ContainerInformation> getAllPeerContainerInformationForLocalSubdomain()
            throws PEtALSWebServiceException {
        Set<ContainerConfiguration> configs = this.topologyService.getContainersConfiguration(null);

        List<ContainerInformation> result = new ArrayList<ContainerInformation>();
        for (ContainerConfiguration cc : configs) {
            try {
                if (cc.isPeer()) {
                    result.add(TopologyConfigurationToTopologyInformation
                            .containerConfigurationToContainerInformation(cc));
                }
            } catch (PetalsException ex) {
                throw new PEtALSWebServiceException(
                        "Can't transform ContainerConfiguration to ContainerInformation");
            }
        }

        return result;
    }

    /**
     * {@inheritDoc}
     */
    public ContainerInformation getContainerInformation(String containerId)
            throws PEtALSWebServiceException {
        ContainerInformation result = null;
        ContainerConfiguration cc = this.topologyService.getContainerConfiguration(containerId);

        if (cc == null) {
            throw new PEtALSWebServiceException("This is not a valid container name ("
                    + containerId + " not found in topology)");
        }
        try {
            result = TopologyConfigurationToTopologyInformation
                    .containerConfigurationToContainerInformation(cc);
        } catch (PetalsException ex) {
            throw new PEtALSWebServiceException(
                    "Can't transform ContainerConfiguration to ContainerInformation : ", ex);
        }

        return result;
    }

    public void addSubdomain(String name, String description, SubdomainMode subdomainMode,
            boolean networkTimeSynchronized) throws PEtALSWebServiceException {
        SubDomainConfiguration newSubdomainConfiguration = new SubDomainConfiguration();
        newSubdomainConfiguration.setName(name);
        newSubdomainConfiguration.setDescription(description);
        newSubdomainConfiguration.setNetworkTimeSynchronized(networkTimeSynchronized);

        if (!this.topologyService.hasValidLocalContainerDynamicTopologyConfiguration()) {
            throw new PEtALSWebServiceException(
                    "No dynamic configuration found on the target container");
        }

        switch (subdomainMode) {
            case FLOODING:
                newSubdomainConfiguration
                        .setMode(org.ow2.petals.topology.generated.SubdomainMode.FLOODING);
                break;
            case MASTERSLAVE:
                newSubdomainConfiguration
                        .setMode(org.ow2.petals.topology.generated.SubdomainMode.MASTER_SLAVE);
                break;
            case STANDALONE:
                newSubdomainConfiguration
                        .setMode(org.ow2.petals.topology.generated.SubdomainMode.STANDALONE);
                break;
            default:
                throw new PEtALSWebServiceException("Unsupported subdomain mode");
        }
        try {
            this.topologyService.addSubdomainConfiguration(newSubdomainConfiguration);
        } catch (PetalsException ex) {
            throw new PEtALSWebServiceException(ex);
        }
    }

    public void removeSubdomain(String subdomainName) throws PEtALSWebServiceException {
        if (!this.topologyService.hasValidLocalContainerDynamicTopologyConfiguration()) {
            throw new PEtALSWebServiceException(
                    "No dynamic configuration found on the target container");
        }

        try {
            this.topologyService.removeSubdomainConfiguration(subdomainName);
        } catch (PetalsException ex) {
            throw new PEtALSWebServiceException(ex.getMessage(), ex);
        }
    }

    public void addSubdomainWithContainers(SubdomainInformation subdomain)
            throws PEtALSWebServiceException {
        if (subdomain == null) {
            throw new PEtALSWebServiceException("No subdomain found");
        }

        if (!this.topologyService.hasValidLocalContainerDynamicTopologyConfiguration()) {
            throw new PEtALSWebServiceException(
                    "No dynamic configuration found on the target container");
        }

        String subdomainName = subdomain.getName();

        // Add the subdomain first
        String description = subdomain.getDescription();
        boolean networkTimeSynchronized = subdomain.isNetworkTimeSynchronized();
        SubdomainMode subdomainMode = subdomain.getSubdomainMode();

        this.addSubdomain(subdomainName, description, subdomainMode, networkTimeSynchronized);

        // Then add the containers (can be quite slow)
        List<ContainerInformation> containers = subdomain.getContainers();

        for (ContainerInformation container : containers) {
            this.addContainerInformation(container, subdomainName, false);
        }
    }

    public SubdomainInformation getSubdomainByName(String subdomainName)
            throws PEtALSWebServiceException {
        SubDomainConfiguration retrievedSubdomain = null;
        SubdomainInformation subdomainInformation = null;

        if (subdomainName == null) {
            throw new PEtALSWebServiceException("No subdomain name found");
        }

        try {
            Set<SubDomainConfiguration> subdomainList = this.topologyService
                    .getSubDomainsConfiguration();
            if (subdomainList != null) {
                for (SubDomainConfiguration subdomainConfiguration : subdomainList) {
                    if (subdomainConfiguration.getName().equals(subdomainName)) {
                        retrievedSubdomain = subdomainConfiguration;
                        break;
                    }
                }
            }

            if (retrievedSubdomain == null) {
                StringBuilder errorSB = new StringBuilder();
                errorSB.append("Can't retrieve subdomain for name " + subdomainName);
            }

            // A subdomain without any containers
            subdomainInformation = TopologyConfigurationToTopologyInformation
                    .subDomainConfigurationToSubdomainInformation(retrievedSubdomain);

            Set<ContainerConfiguration> registeredContainersConfigurations = this.topologyService
                    .getContainersConfigurationsForLocalSubdomain();
            List<ContainerInformation> registeredContainersInformations = new ArrayList<ContainerInformation>();

            for (ContainerConfiguration containerConfiguration : registeredContainersConfigurations) {
                registeredContainersInformations.add(TopologyConfigurationToTopologyInformation
                        .containerConfigurationToContainerInformation(containerConfiguration));
            }

            subdomainInformation.setContainers(registeredContainersInformations);

        } catch (PetalsException ex) {
            throw new PEtALSWebServiceException(ex);
        }

        return subdomainInformation;
    }

    public DomainInformation getDomainInformation() throws PEtALSWebServiceException {
        DomainInformation domainInformation = new DomainInformation();

        try {
            domainInformation.setName(this.topologyService.getDomainConfiguration().getName());
            domainInformation.setDescription(this.topologyService.getDomainConfiguration()
                    .getDescription());
            domainInformation.setJndiInformation(this.getJndiInformation());
            domainInformation.setMode(this.domainModeTransformer(this.topologyService
                    .getDomainConfiguration().getMode()));

            List<SubdomainInformation> subdomainsInformations = new ArrayList<SubdomainInformation>();

            for (SubDomainConfiguration subdomainConfiguration : this.topologyService
                    .getSubDomainsConfiguration()) {
                SubdomainInformation subInfo = TopologyConfigurationToTopologyInformation
                        .subDomainConfigurationToSubdomainInformation(subdomainConfiguration);
                subdomainsInformations.add(subInfo);
            }

            domainInformation.setSubdomains(subdomainsInformations);

        } catch (PetalsException ex) {
            throw new PEtALSWebServiceException(ex);
        }

        return domainInformation;
    }

    private org.ow2.petals.kernel.ws.api.to.DomainInformation.DomainMode domainModeTransformer(
            org.ow2.petals.kernel.configuration.DomainConfiguration.DomainMode domainMode)
            throws PetalsException {

        switch (domainMode) {
            case STATIC:
                return org.ow2.petals.kernel.ws.api.to.DomainInformation.DomainMode.STATIC;
            case STANDALONE:
                return org.ow2.petals.kernel.ws.api.to.DomainInformation.DomainMode.STANDALONE;
            default:
                throw new PetalsException("Invalid domain mode");
        }
    }
}
